# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
# Copyright (C) 2021 Graz University of Technology.
# Copyright (C) 2021 TU Wien.
# Copyright (C) 2022 Northwestern University.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community permissions."""

from invenio_records_permissions.generators import AnyUser, \
    AuthenticatedUser, Disable, SystemProcess
from invenio_records_permissions.policies import BasePermissionPolicy

from .generators import AllowedMemberTypes, CommunityManagers, \
    CommunityMembers, CommunityOwners, CommunitySelfMember, IfPolicyClosed, \
    IfRestricted


# Permission Policy
class CommunityPermissionPolicy(BasePermissionPolicy):
    """Permissions for Community CRUD operations."""

    # Community
    can_create = [AuthenticatedUser(), SystemProcess()]

    can_read = [
        IfRestricted(
            'visibility',
            then_=[CommunityMembers()],
            else_=[AnyUser()]),
        ]

    can_update = [CommunityOwners(), SystemProcess()]

    can_delete = [CommunityOwners(), SystemProcess()]

    can_search = [AnyUser(), SystemProcess()]

    can_search_user_communities = [AuthenticatedUser(), SystemProcess()]

    can_rename = [CommunityOwners(), SystemProcess()]

    can_submit_record = [
        IfPolicyClosed(
            'record_policy',
            then_=[CommunityMembers(), SystemProcess()],
            else_=[IfRestricted(
                'visibility',
                then_=[CommunityMembers()],
                else_=[AuthenticatedUser()]),
            ],
        ),
    ]

    can_members_add = [
        CommunityManagers(),
        AllowedMemberTypes('group'),
        SystemProcess(),
    ]

    can_members_invite = [
        CommunityManagers(),
        AllowedMemberTypes('user', 'email'),
        SystemProcess(),
    ]

    can_members_search = [
        CommunityMembers(),
        SystemProcess(),
    ]

    can_members_search_public = [
        IfRestricted(
            'visibility',
            then_=[CommunityMembers()],
            else_=[AnyUser()]
        ),
        SystemProcess(),
    ]

    # Ability to use membership update api
    can_members_bulk_update = [
        CommunityMembers(),
        SystemProcess(),
    ]

    can_members_bulk_delete = can_members_bulk_update

    # Ability to update a single membership
    can_members_update = [
        CommunityManagers(),
        CommunitySelfMember(),
        SystemProcess(),
    ]

    # Ability to delete a single membership
    can_members_delete = can_members_update



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration."""

from invenio_communities.communities.services import facets
from flask_babelex import lazy_gettext as _

COMMUNITIES_ROUTES = {
    'frontpage': '/communities',
    'search': '/communities/search',
    'new': '/communities/new',
    'details': '/communities/<pid_value>',
    'settings': '/communities/<pid_value>/settings',
    'settings_privileges': '/communities/<pid_value>/settings/privileges',
    'members': '/communities/<pid_value>/members',
    'invitations': '/communities/<pid_value>/invitations'
}
"""Communities ui endpoints."""

COMMUNITIES_ENABLED = True
"""Config to enable/disable communities blueprints."""

COMMUNITIES_FACETS = {
    'type': {
        'facet': facets.type,
        'ui': {
            'field': 'type',
        }
    },
    'visibility': {
        'facet': facets.visibility,
        'ui': {
            'field': 'visibility',
        }
    }
}
"""Available facets defined for this module."""


COMMUNITIES_ROLES = {
    'reader': dict(
        title=_('Reader'),
        description=_('Can view restricted records'),
    ),
    'curator': dict(
        title=_('Curator'),
        description=_('Can edit and view restricted records'),
    ),
    'manager': dict(
        title=_('Manager'),
        description=_('Can manage members, curate records and view restricted records'),
        can_manage_roles=['manager', 'curator', 'reader'],
    ),
    'owner': dict(
        title=_('Owner'),
        description=_('Full administrative access to the entire community.'),
        can_manage_roles=['owner', 'manager', 'curator', 'reader'],
        is_owner=True
    ),
}
"""Community roles."""
